# -*- coding: utf-8 -*-

# Netsphere Financial Front / ネットスフィア経理フロント
# Copyright (C) 2007-2009,2011 HORIKAWA Hisashi. All rights reserved.
#     http://www.nslabs.jp/bside/
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


require 'validating_base'

class SearchBox < ValidatingBase
  attr_accessor :text, :code
end

# ユーザ（社員）
class UsersController < ApplicationController
  before_filter :login_required
  
  # GETs should be safe (see http://www.w3.org/2001/tag/doc/whenToUseGet.html)
  verify :method => :post, :only => [ :destroy, :create, :update ],
         :redirect_to => { :action => :list }
  
  # 検索画面
  def index
  end
  
  # 検索結果
  def search
    @search = SearchBox.new(params[:search])
    if @search.text.to_s.strip == '' && @search.code.to_s.strip == ''
      @search.errors.add_to_base '検索テキストを指定してください。'
      render :action => 'index'
      return
    end

    c = []; v = []
    if @search.text.to_s.strip != ''
      text = '%' + normalize_kc(@search.text.strip).upcase + '%' # PostgreSQL only. TODO:fixme
      c << '(upper(name) LIKE ? OR kana_name LIKE ? OR upper(email) LIKE ?)'
      v << text << text << text
    end
    if @search.code.to_s.strip != ''
      c << 'worker_code = ?'
      v << @search.code
    end
    @users = User.find :all,
                       :conditions => [c.join(' AND '), *v],
                       :limit => 40
  end
  
  # 一覧
  def list
    @workers = User.paginate :page => params[:page], :per_page => 25,
                             :order => 'worker_code'
  end

  def show
    @worker = User.find params[:id]
    @exist_gl = GeneralLedger.find :first,
                                   :conditions => ['created_by = ?', @worker.id]
  end

  def new
    @worker = User.new
  end

  def create
    @worker = User.new(params[:worker])
    @worker.active = 1
    if @worker.save
      flash[:notice] = 'Worker was successfully created.'
      redirect_to :action => 'show', :id => @worker
    else
      render :action => 'new'
    end
  end

  def edit
    @worker = User.find params[:id]
  end

  def update
    if !current_user.has_role?(:hrd)
      redirect_to '/'
      return
    end

    @worker = User.find(params[:id])
    if @worker.update_attributes(params[:worker])
      flash[:notice] = 'Worker was successfully updated.'
      redirect_to :action => 'show', :id => @worker
    else
      render :action => 'edit'
    end
  end

  # 削除
  def destroy
    u = User.find params[:id]
    User.transaction {
      UsersDivision.delete_all ['user_id = ?', u.id]
      u.destroy
    }
    flash[:notice] = "ユーザ #{u.worker_code} #{u.name} を削除しました。"
    redirect_to :action => 'index'
  end
  
  # 取り扱える会計部門の編集画面
  def edit_divisions
    @worker = User.find params[:id]
    @divisions = @worker.divisions.find :all, 
                      :conditions => ['users_divisions.effective_day IS NULL']
  end

  
  # 取り扱える会計部門を更新
  def update_divisions
    require 'set'
    @worker = User.find params[:id]
    new_divs = params[:div].collect {|v| v.to_i}.delete_if {|v| v == 0}
    UsersDivision.transaction {
      # 一度全部消して、新たに作成する
      UsersDivision.delete_all ["user_id = ? AND effective_day IS NULL", @worker.id]
      new_divs.each_with_index {|d, i|
        r = UsersDivision.new :user => @worker,
                              :division_id => d,
                              :effective_day => nil,
                              :default_pair => 0
        r.save!
      }
    } # of transaction
    flash[:notice] = "Worker was successfully updated."
    redirect_to :action => 'show', :id => @worker
  end


  # 社内での経歴の編集 (画面)
  def edit_division_history
    @worker = User.find params[:id]
  end
  
  # 異動情報を更新
  def update_division_history
    @worker = User.find params[:id]
    lines = params[:lines].to_i
    raise "internal error" if lines == 0

    new_divs = []
    default_pair = nil
    for i in 0..(lines - 1) do
      param = params["division#{i}"]
      endenable = param[:endenable]
      param.delete(:endenable)

      ud = UsersDivision.new param
      if ud.division_id
        ud.user_id = @worker.id
        ud.end_day = nil if !endenable
        if !ud.end_day && !default_pair
          ud.default_pair = 1
          default_pair = ud
        else
          ud.default_pair = 0
        end
        new_divs << ud
      end
    end

    UsersDivision.transaction {
      UsersDivision.delete_all ['user_id = ? AND effective_day IS NOT NULL', @worker.id]
      new_divs.each {|ud|
        ud.save!
      }
    }
    flash[:notice] = "異動情報を更新しました。"
    redirect_to :action => 'show', :id => @worker
  end
  
  def new_qualification
    @worker = User.find params[:id]
    @uq = UsersQualification.new
  end
  
  def create_qualification
    @worker = User.find params[:id]
    @uq = UsersQualification.new params[:uq]

    @uq.user_id = @worker.id
    @uq.expire_day = nil if !params[:onend]
    
    if @uq.save
      flash[:notice] = "資格情報を登録しました。"
      redirect_to :action => 'show', :id => @worker
    else
      render :action => 'new'
    end
  end
  
  # 資格の編集 (画面)
  def edit_qualification
    @worker = User.find params[:id]
    @uq = UsersQualification.find params[:uq]
  end
  
  # 資格の更新
  def update_qualification
    @worker = User.find params[:id]
    @uq = UsersQualification.find params[:uq][:id]
    @uq.attributes = params[:uq]
    @uq.expire_day = nil if !params[:onend]
    
    if @uq.save
      flash[:notice] = "資格情報を更新しました。"
      redirect_to :action => 'show', :id => @worker
    else
      render :action => 'edit'
    end
  end

  def destroy_qualification
    @worker = User.find params[:id]
    @uq = UsersQualification.find params[:uq]

    @uq.destroy
    flash[:notice] = "資格情報を削除しました。"
    redirect_to :action => 'show', :id => @worker
  end
end
