# -*- coding: utf-8 -*-

# Netsphere Financial Front / ネットスフィア経理フロント
# Copyright (C) 2007-2009,2011 HORIKAWA Hisashi. All rights reserved.
#     http://www.nslabs.jp/bside/
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


require "ostruct" # OpenStruct


# (購買) 請求書
class Invoice < Document
  belongs_to :company_setting, :class_name => "Company"
  belongs_to :partner
  belongs_to :division
  belongs_to :bank_account
  belongs_to :en_bank_account
  belongs_to :currency
  belongs_to :created_by, :foreign_key => 'created_by', :class_name => 'User'
  has_many :invoice_details, :dependent => :destroy
  has_many :attach_file_refs
  has_many :attach_files, :through => :attach_file_refs
  has_many :imports_invoices
  has_many :import_declarations, :through => :imports_invoices

  SIGNED_OFF = 10
  
  before_save :normalize_string

  validate :check_fields
  validate :check_fields_on_create, :on => :create
  validate :check_fields_on_update, :on => :update

  
  # for before_save
  def normalize_string
    self.invoice_no = normalize_kc invoice_no
  end


  # 請求金額の合計を更新。save は呼び出し側ですること。
  # 正規化された金額, 輸入消費税含む。
  def update_amount_total
    a = InvoiceDetail.find :first,
                           :select => 'sum(amount) AS sum_amount',
                           :conditions => ['invoice_id = ?', id]
    
    self.amount_total = a.sum_amount.to_i + amount_of_import_vat

    return self
  end


  # 関連づけられている輸入消費税（地方税含む。）の合計金額
  def amount_of_import_vat
    v = ImportDeclaration.find :first,
             :select => 'sum(consumption_tax) AS sum_vat, sum(vat_local) AS sum_local_vat',
             :joins => 'LEFT JOIN imports_invoices ON import_declarations.id = imports_invoices.import_declaration_id',
             :conditions => ['imports_invoices.invoice_id = ?', id]

    return v ? v.sum_vat.to_i + v.sum_local_vat.to_i : 0
  end


  # 作成, 更新共通
  # for validate
  def check_fields
    # errors.add_to_base('明細が1行もありません。') if invoice_details.empty? && import_declarations.empty?
    
    if due_day.is_a?(Date) && buy_day.is_a?(Date) && (due_day < buy_day)
      errors.add('buy_day', '請求締め日は支払日より前にしてください')
    end
  end


  # for validate
  def check_fields_on_create
    if invoice_no
      a = Invoice.find :first,
               :conditions => ['partner_id = ? AND invoice_no = ?',
                               partner_id, invoice_no]
      errors.add('invoice_no', '請求書番号が重複しています。') if a
    end
  end

  
  # for validate
  def check_fields_on_update
    if invoice_no
      a = Invoice.find :first,
               :conditions => ['partner_id = ? AND invoice_no = ? AND id <> ?',
                               partner_id, invoice_no, id]
      errors.add('invoice_no', '請求書番号が重複しています。') if a
    end
  end


  # 購買請求書 -> 仕訳
  def make_entries company, entry_day
    result = []
    invoice_details.each {|detail|
      amount, rate = get_gl_amount company, entry_day, detail.amount, currency
      vat_code = if detail.vat_code
                   detail.vat_code # for compat.
                 else
                   x_ = detail.account_title.get_real_vat( detail.vat_id, 
                                                      company.country )
                   x_ ? x_.tax_code : "40" # for compat.
                 end

      line = {
        :entry_day => entry_day,
        :entry_no => 10000 + id,  # TODO:決め打ち。fixme
        
        # 借方
        :dr_account_title => detail.account_title,
        :dr_division => detail.division,
        :dr_partner_code => (detail.account_title.req_partner ? partner.zaimu_id : nil),

        # 貸方
        :cr_account_title => detail.account_title.payable,
        :cr_division => nil,
        :cr_partner_code => (bank_account ? bank_account.zaimu_ac : partner.zaimu_id),

        :amount => amount,
        :vat => vat_code,
        :due_day => (bank_account ? due_day : nil),
        :created_by => created_by,
      }
      # 摘要
      if currency.id == company.functional_currency.id
        line[:remarks] = [detail.division.name, partner.name,
                            detail.remarks].join('/')
      else
        line[:remarks] = [detail.division.name, partner.name,
          "#{detail.remarks} #{format_amount(detail.amount, currency.exponent)}#{currency.code}@#{rate}"].join('/')
      end
      line[:remarks] << "/稟議#" + approval if !approval.blank?

      result << line
    }

    # 輸入消費税    
    vat_payment_ac = AccountTitle.vat_payment
    import_declarations.each {|imp|
      line = {
        :entry_day => entry_day,
        :entry_no => 10000 + id,  # TODO:決め打ち。fixme

        # 借方
        :dr_account_title => vat_payment_ac,
        :dr_division => nil,
        :dr_partner_code => nil,

        # 貸方
        :cr_account_title => AccountTitle.find(vat_payment_ac.payable_id),
        :cr_division => nil,
        :cr_partner_code => (bank_account ? bank_account.zaimu_ac : partner.zaimu_id),

        :amount => imp.consumption_tax + imp.vat_local,
        :vat => 40,
        :due_day => (bank_account ? due_day : nil),
        :created_by => created_by,
        :remarks => "輸入消費税 ID#" + [imp.format_declaration_number, imp.permit_date, imp.exporter_name].join('/')
      }
      result << line
    }
    return result
  end


  # 未払残額 (合計, 輸入消費税含む。)
  # 未承認の出金伝票でも、未払い残額は減らす
  def remain_amount
    r = Payment.find :first,
           :select => "sum(payments.amount) AS amount_payed",
           :joins => "LEFT JOIN payment_slips ON payments.payment_slip_id = payment_slips.id",
           :conditions => ["payment_slips.state <> 30 AND invoice_id = ?", id]

    return r ? amount_total - r.amount_payed.to_i : amount_total
  end


  # 勘定科目ごとの未払残額
  # @return {勘定科目id => 金額}
  def remain_ac_amounts
    inv_amounts = InvoiceDetail.find :all,
            :select => 'sum(amount) AS sum_amount, drac.payable_id',
            :joins => "LEFT JOIN account_titles AS drac ON invoice_details.account_title_id = drac.id",
            :conditions => ['invoice_id = ?', id],
            :group => "drac.payable_id"
    vat_amount = amount_of_import_vat.to_i

    if vat_amount != 0
      f = false
      inv_amounts.each do |dtl|
        raise RuntimeError if !dtl.payable_id
        if dtl.payable_id == AccountTitle.vat_payment.payable_id
          dtl.sum_amount = dtl.sum_amount.to_i + vat_amount
          f = true
          break
        end
      end
      if !f
        inv_amounts << (OpenStruct.new :sum_amount => vat_amount, 
                            :payable_id => AccountTitle.vat_payment.payable_id)
      end
    end

    result = {}
    inv_amounts.each do |dtl|
      payed = Payment.find :first,
                 :select => "sum(payments.amount) AS amount_payed",
                 :joins => "LEFT JOIN payment_slips ON payments.payment_slip_id = payment_slips.id",
                 :conditions => ["(payment_slips.state = 10 OR payment_slips.state = 20) AND invoice_id = ? AND account_title_id = ?", id, dtl.payable_id]

      if payed && payed.amount_payed
        result[dtl.payable_id] = dtl.sum_amount.to_i - payed.amount_payed.to_i
      else
        result[dtl.payable_id] = dtl.sum_amount.to_i
      end
    end

    return result
  end


  # 最初の明細行の内容
  # @return [String] 内容
  def first_detail_remarks
    d = invoice_details.first
    if d
      return d.remarks
    else
      d = import_declarations.first
      if d
        return "#{d.exporter_name} ID##{d.declaration_number}"
      else
        return "(明細なし)"
      end
    end
  end


  # 仕訳で利用する円建ての金額、そのレート
  # payment_slip で利用される
  def amount_total_local_and_rate
    entry_day = journal_day || [buy_day, company_setting.fixed_gl_day + 1].max
    return get_yen_amount(entry_day, amount_total, currency)
  end

end # class Invoice
