# -*- coding:utf-8 -*-

# Netsphere Financial Front / ネットスフィア経理フロント
# Copyright (C) 2007-2009,2011 HORIKAWA Hisashi. All rights reserved.
#     http://www.nslabs.jp/bside/
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


# 出金伝票
class PaymentSlip < Document
  belongs_to :company
  belongs_to :created_by, :foreign_key => "create_user_id", 
                                                      :class_name => "User"

  has_many :payments
  has_many :payment_banks

  SIGNED_OFF = 10


  # 仕訳を生成
  # @return [Array] 仕訳の配列
  #
  # 請求書の通貨と出金口座の通貨が同じ
  #    (1) 通貨=JPY || 債務の発生月と支払月が同じ
  #
  def make_entries company, entry_day
    return [] if payments.size == 0 || payment_banks.size == 0

    inv_currency = payments.first.invoice.currency
    result = []

    if payment_banks.size == 1 &&
             inv_currency == payment_banks.first.our_bank.currency
      # 出金口座が一つ => 場合によっては諸口は通さない
      payment_bank = payment_banks.first
      cash_rate_local = get_yen_amount(entry_day, 
                                       payment_bank.cash_amount, 
                                       payment_bank.our_bank.currency)[1]
      payable_total_foreign = 
                 ( payments.inject(0.0) do |sum, r| sum + r.amount.to_f end )

      payments.each do |payable|
        next if payable.amount == 0
        
        inv = payable.invoice
        inv_rate_local = inv.amount_total_local_and_rate[1]

        if inv_rate_local == cash_rate_local
          # 左右で同じレート => 諸口を通さない
          e = {
            :entry_day => entry_day, :entry_no => 51000 + id,
            :dr_account_title => payable.account_title,
            :dr_division => nil, :dr_partner_code => inv.partner.zaimu_id,
            :cr_account_title => payment_bank.our_bank.account_title,
            :cr_division => nil, :cr_partner_code => nil,
            :amount => ((payable.amount.to_f / 10 ** inv_currency.exponent) * inv_rate_local + 0.5).floor,
            :vat => nil, :due_day => nil,
            :created_by => created_by,
            :remarks => [inv.partner.name, inv.first_detail_remarks, 
                         "支払い"].join("/")
          }
          if inv_currency.code != "JPY"
            e[:remarks] << " " <<
              format_amount(payable.amount, inv_currency.exponent) <<
              inv_currency.code << "@" << inv_rate_local.to_s
          end
          result << e
        else
          # 通貨は同じだが債務の発生月が違う => 諸口を通す

          # 請求書の金額ではない
          payable_amount_local = 
             ( (payable.amount.to_f / 10 ** inv_currency.exponent) * 
                                                   inv_rate_local + 0.5).floor
          e = {
            :entry_day => entry_day, :entry_no => 51000 + id,
            :dr_account_title => payable.account_title,
            :dr_division => nil, :dr_partner_code => inv.partner.zaimu_id,
            :cr_account_title => nil,
            :cr_division => nil, :cr_partner_code => nil,
            :amount => payable_amount_local,
            :vat => nil, :due_day => nil,
            :created_by => created_by,
            :remarks => [inv.partner.name, inv.first_detail_remarks, 
                         "支払い"].join("/")
          }
          if inv_currency.code != "JPY"
            e[:remarks] << " " <<
              format_amount(payable.amount, inv_currency.exponent) <<
              inv_currency.code << "@" << inv_rate_local.to_s
          end
          result << e

          # raise "cash_amount = #{payment_bank.cash_amount.to_f}, payable_amount = #{payable.amount.to_f}, payable_total_foreign = #{payable_total_foreign.to_f}" # DEBUG
          cash_amount_foreign = (payment_bank.cash_amount.to_f / 
                           10 ** payment_bank.our_bank.currency.exponent) *
                       payable.amount.to_f / payable_total_foreign.to_f
          cash_amount_local = (cash_amount_foreign * cash_rate_local + 0.5).floor
          e = {
            :entry_day => entry_day, :entry_no => 51000 + id,
            :dr_account_title => nil,
            :dr_division => nil, :dr_partner_code => nil,
            :cr_account_title => payment_bank.our_bank.account_title,
            :cr_division => nil, :cr_partner_code => nil,
            :amount => cash_amount_local,
            :vat => nil, :due_day => nil,
            :created_by => created_by,
            :remarks => [inv.partner.name, inv.first_detail_remarks, 
                         "支払い"].join("/")
          }
          if payment_bank.our_bank.currency.code != "JPY"
            e[:remarks] << " " <<
                    format_amount(payment_bank.cash_amount.to_f * 
                            (payable.amount.to_f / payable_total_foreign.to_f), 
                                  payment_bank.our_bank.currency.exponent) <<
                    payment_bank.our_bank.currency.code << "@" << 
                    ((cash_amount_local.to_f / cash_amount_foreign * 10000 + 0.5).floor.to_f / 10000).to_s
          end
          result << e
          
          e = {
            :entry_day => entry_day, :entry_no => 51000 + id,
            :dr_account_title => nil,
            :dr_division => nil, :dr_partner_code => nil,
            :cr_account_title => AccountTitle.currency_gain,
            :cr_division => company.bank_division,
            :cr_partner_code => nil,
            :amount => payable_amount_local - cash_amount_local,
            :vat => 40, :due_day => nil,
            :created_by => created_by,
            :remarks => [inv.partner.name, inv.first_detail_remarks, 
                         "為替差"].join('/') 
          }
          result << e
        end
      end # payments.each
    else
      # 出金口座が二つ => 伝票の仕訳全体で諸口を通す
      payable_total = 0
      payments.each do |payable|
        inv = payable.invoice
        inv_rate_local = inv.amount_total_local_and_rate[1]

        # 請求書の金額ではない
        payable_amount_local = 
                  ( (payable.amount.to_f / 10 ** inv_currency.exponent) *  
                                                  inv_rate_local + 0.5).floor
        e = {
          :entry_day => entry_day, :entry_no => 51000 + id,
          :dr_account_title => payable.account_title,
          :dr_division => nil, :dr_partner_code => inv.partner.zaimu_id,
          :cr_account_title => nil,
          :cr_division => nil, :cr_partner_code => nil,
          :amount => payable_amount_local,
          :vat => nil, :due_day => nil,
          :created_by => created_by,
          :remarks => [inv.partner.name, inv.first_detail_remarks, 
                         "支払い"].join("/")
        }
        if inv_currency.code != "JPY"
          e[:remarks] << " " <<
              format_amount(payable.amount, inv_currency.exponent) <<
              inv_currency.code << "@" << inv_rate_local.to_s
        end
        result << e
        payable_total += payable_amount_local
      end

      cash_total = 0
      payment_banks.each do |payment_bank|
        cash_rate_local = get_yen_amount(entry_day, 
                                       payment_bank.cash_amount, 
                                       payment_bank.our_bank.currency)[1]
        cash_amount_local = 
                ( (payment_bank.cash_amount.to_f / 10 ** payment_bank.our_bank.currency.exponent) * cash_rate_local + 0.5).floor
        e = {
          :entry_day => entry_day, :entry_no => 51000 + id,
          :dr_account_title => nil,
          :dr_division => nil, :dr_partner_code => nil,
          :cr_account_title => payment_bank.our_bank.account_title,
          :cr_division => nil, :cr_partner_code => nil,
          :amount => cash_amount_local,
          :vat => nil, :due_day => nil,
          :created_by => created_by,
          :remarks => "支払い"
        }
        if payment_bank.our_bank.currency.code != "JPY" ||
                  payment_bank.our_bank.currency.code != inv_currency.code
          e[:remarks] << " " <<
                    format_amount(payment_bank.inv_amount, 
                                  inv_currency.exponent) <<
                    inv_currency.code << "@" << 
                    ((cash_amount_local.to_f / (payment_bank.inv_amount.to_f / 10 ** inv_currency.exponent) * 10000 + 0.5).floor.to_f / 10000).to_s
        end
        result << e
        cash_total += cash_amount_local
      end

      # 為替差損益 (合計)
      if (payable_total - cash_total) != 0
        e = {
          :entry_day => entry_day, :entry_no => 51000 + id,
          :dr_account_title => nil,
          :dr_division => nil, :dr_partner_code => nil,
          :cr_account_title => AccountTitle.currency_gain,
          :cr_division => company.bank_division,
          :cr_partner_code => nil,
          :amount => payable_total - cash_total,
          :vat => 40, :due_day => nil,
          :created_by => created_by,
          :remarks => "為替差"
        }
        result << e
      end
    end

    return result
  end


  # 支払う相手先の一覧
  def invoice_partners
    partners = Partner.find_by_sql [<<EOF, self.id]
SELECT DISTINCT partners.* 
    FROM payments
         LEFT JOIN invoices ON payments.invoice_id = invoices.id 
         LEFT JOIN partners ON invoices.partner_id = partners.id
    WHERE payment_slip_id = ?
EOF
    return partners
  end
end

