# -*- coding: utf-8 -*-

# Netsphere Financial Front / ネットスフィア経理フロント
# Copyright (C) 2007-2009,2011 HORIKAWA Hisashi. All rights reserved.
#     http://www.nslabs.jp/bside/
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


# 販売請求書
class SalesInvoice < Document
  has_many :si_details, :dependent => :destroy
  has_many :si_journals, :dependent => :destroy
  has_many :si_banks, :dependent => :destroy

  has_many :our_banks, :through => :si_banks
  
  belongs_to :company_setting, :class_name => "Company"

  belongs_to :partner
  belongs_to :division   # 所管部門
  belongs_to :currency
  belongs_to :created_by, :foreign_key => 'create_user_id', :class_name => 'User'
  belongs_to :issued_by, :foreign_key => 'issued_by', :class_name => 'User'
  belongs_to :receivable_account, :foreign_key => 'receivable_account_id',
                                  :class_name => 'AccountTitle'
  
  before_save :update_dependency

  DELIVERY_TYPE_NAMES = [nil, '担当者持参', '経理から郵送']
  SIGNED_OFF = 10
  
  validates_presence_of :delivery
  validates_presence_of :partner
  validates_presence_of :division
  validate :check_fields


  # for validate
  def check_fields
    errors.add_to_base('明細が1行もありません。') if si_details.empty?
    errors.add_to_base('仕訳 (計上) が1行もありません。') if si_journals.empty?
  end


  # 戻り値は (10 ** currency.exponent) した額
  def ex_vat_amount
    ex_total = 0
    si_details.each {|r|
      ex_total += r.amount if r.vat_code == 20
    }
    return (ex_total * 0.05).floor
  end
  
  # 債権科目と請求額合計を更新
  def update_dependency
    self.receivable_account = get_receivable_of_heavy_account(si_journals)
    raise "AR setting is missing!" if !self.receivable_account
    self.amount_total = si_journals.inject(0) {|result, item| result += item.amount}
  end
  private :update_dependency
  
  # 一番金額の大きな科目 (主たる科目) に対応する債権科目
  def get_receivable_of_heavy_account(journals)
    if !journals || journals.size < 1
      return nil # valid でエラー
    end

    a = {}
    journals.each {|r|
      a[r.account_title_id] = (a[r.account_title_id] || 0) + r.amount
    }
    return AccountTitle.find(a.to_a.sort_by {|x| x[1]}.last[0]).receivable
  end
  private :get_receivable_of_heavy_account


  # 未入金の額（残額）
  # 未承認の入金伝票でも、未入金金額は減らす
  def remain_amount
    r = Receiving.find :first,
             :select => 'sum(receivings.amount) as received', # , sum(charge) as charge',
             :joins => "LEFT JOIN receiving_slips ON receivings.receiving_slip_id = receiving_slips.id",
             :conditions => ['receiving_slips.state <> 30 AND sales_invoice_id = ?', id]
    if r && r.received
      return amount_total - r.received.to_i # - r.charge.to_i
    else
      return amount_total
    end
  end
  

  # 販売請求書 -> 仕訳
  def make_entries company, entry_day
    result = []
    si_journals.each {|line|
      amount, rate = get_yen_amount(entry_day, line.amount, currency)
      vat_code = if line.vat_code
                   line.vat_code # for compat.
                 else
                   x_ = line.account_title.get_real_vat( line.vat_id, 
                                                      company.country )
                   x_ ? x_.tax_code : "40" # for compat.
                 end

      e = {
        :entry_day => entry_day,
        :entry_no => 40000 + id,  # TODO:またもや決め打ち。fixme

        :dr_account_title => receivable_account,
        :dr_division => nil,
        :dr_partner_code => partner.zaimu_id,

        :cr_account_title => line.account_title,
        :cr_division => line.account_title.req_partner ? nil : line.division,
        :cr_partner_code => line.account_title.req_partner ? partner.zaimu_id : nil,

        :amount => amount,
        :vat => vat_code,
        :due_day => due_day,
        :created_by => created_by
      }
      e[:remarks] = [line.division.name, partner.name, line.remarks].join('/')
      if currency.code != company.functional_currency.code
        # raise "#{currency.code.inspect}, #{rate.inspect}" # DEBUG
        e[:remarks] << " " <<
                       format_amount(line.amount, currency.exponent) <<
                       currency.code << "@" << rate.to_s
      end
      result << e
    }
    return result
  end
  

  # 仕訳にした円建ての金額, レート
  def yen_amount_total
    entry_day = journal_day || [sell_day, company_setting.fixed_gl_day + 1].max
    return get_yen_amount(entry_day, amount_total, currency)
  end


  # current_user が編集してもよいか
  # TODO: 会計システム連動後も編集できるようにしているが、考えどころ。
  def editable_by user
    state < Invoice::SIGNED_OFF || (user.has_role?(:keiri) && state < 30)
  end
end
